{% extends 'auth/base-site-user.html' %}
{% from 'auth/form-groups.html' import edit_group_form %}


{% block content2 %}

<h1 class="mt-3 mb-4">Edit Group <em>{{ group.name }}</em></h1>

    <div class="bg-white rounded border p-3 my-3">

        {{ edit_group_form(form, 'auth.edit_group', group.name) }}

    </div>

{% endblock %}